@@ -1,11 +1,13 @@
 from abc import abstractmethod
 from typing import Type
 
-from terminusgps.wialon import flags
+from terminusgps.wialon import constants, flags
 from terminusgps.wialon.session import WialonSession
 
 
 class WialonBase:
+    """Base class for Wialon objects."""
+
     def __init__(
         self, id: str | int | None, session: WialonSession, *args, **kwargs
     ) -> None:
@@ -18,7 +20,7 @@ class WialonBase:
         :type id: :py:obj:`str` | :py:obj:`int` | :py:obj:`None`
         :param session: A valid Wialon API session.
         :type session: :py:obj:`~terminusgps.wialon.session.WialonSession`
-        :raises ValueError: If ``id`` was a string that contained non-digit values.
+        :raises ValueError: If ``id`` wasn't a digit.
         :returns: Nothing.
         :rtype: :py:obj:`None`
 
@@ -40,6 +42,11 @@ class WialonBase:
         """Returns the Wialon object type and parameters used to initialize it."""
         return f"{type(self).__name__}(id={self.id}, session={self.session.id})"
 
+    @abstractmethod
+    def create(self, *args, **kwargs) -> int | None:
+        """Creates a Wialon object and returns its id."""
+        raise NotImplementedError("Subclasses must implement this method.")
+
     def populate(self) -> None:
         """
         Retrieves and saves the latest data for the item from Wialon.
@@ -59,6 +66,191 @@ class WialonBase:
             self._hw_type = item.get("cls")
             self._access_lvl = item.get("uacl")
 
+    def has_access(self, other: Type["WialonBase"]) -> bool:
+        """
+        Checks if this Wialon object has access to ``other``.
+
+        :type: :py:obj:`bool`
+
+        """
+        response = self.session.wialon_api.core_check_accessors(
+            **{"items": [other.id], "flags": False}
+        )
+        return True if self.id in response.keys() else False
+
+    def set_measurement_unit(
+        self,
+        unit: constants.WialonMeasurementUnit = constants.WialonMeasurementUnit.US,
+        convert: bool = False,
+    ) -> None:
+        """
+        Sets the Wialon object's measurement unit.
+
+        :param unit: A Wialon measurement unit. Default is :py:obj:`~terminusgps.wialon.constants.WialonMeasurementUnit.US`.
+        :type unit: :py:obj:`~terminusgps.constants.WialonMeasurementUnit`
+        :param convert: Whether or not to convert the object's measurements before setting the new unit.
+        :type convert: :py:obj:`bool`
+        :raises WialonError: If something went wrong with a Wialon API call.
+        :returns: Nothing.
+        :rtype: :py:obj:`None`
+
+        """
+        self.session.wialon_api.item_update_measure_units(
+            **{"itemId": str(self.id), "type": unit, "flags": int(convert)}
+        )
+
+    def rename(self, new_name: str) -> None:
+        """
+        Renames the Wialon object to the new name.
+
+        :param new_name: A new name.
+        :type new_name: :py:obj:`str`
+        :raises WialonError: If something went wrong with a Wialon API call.
+        :returns: Nothing.
+        :rtype: :py:obj:`None`
+
+        """
+        self.session.wialon_api.item_update_name(
+            **{"itemId": self.id, "name": new_name}
+        )
+        self._name = new_name
+
+    def delete(self) -> None:
+        """
+        Deletes the Wialon object.
+
+        :raises WialonError: If something went wrong with a Wialon API call.
+        :returns: Nothing.
+        :rtype: :py:obj:`None`
+
+        """
+        self.session.wialon_api.item_delete_item(**{"itemId": self.id})
+
+    def update_pfield(self, key: constants.WialonProfileField, value: str) -> None:
+        """
+        Updates a profile field by key.
+
+        :param key: A profile field key (name).
+        :type key: :py:obj:`~terminusgps.wialon.constants.WialonProfileField`
+        :param value: A string.
+        :type value: :py:obj:`str`
+        :raises WialonError: If something went wrong with a Wialon API call.
+        :returns: Nothing.
+        :rtype: :py:obj:`None`
+
+        """
+        self.session.wialon_api.item_update_profile_field(
+            **{"itemId": self.id, "n": key.value, "v": value}
+        )
+
+    def update_afield(self, key: str, value: str) -> None:
+        """
+        Updates an admin field by key.
+
+        :param key: An admin field key (name).
+        :type key: :py:obj:`str`
+        :param value: A string.
+        :type value: :py:obj:`str`
+        :raises WialonError: If something went wrong with a Wialon API call.
+        :returns: Nothing.
+        :rtype: :py:obj:`None`
+
+        """
+        field_id: int | None = self._get_afield_id(key)
+
+        if field_id is not None:
+            self.session.wialon_api.item_update_admin_field(
+                **{
+                    "itemId": self.id,
+                    "id": field_id,
+                    "callMode": "update",
+                    "n": key,
+                    "v": value,
+                }
+            )
+        else:
+            self.session.wialon_api.item_update_admin_field(
+                **{"itemId": self.id, "callMode": "create", "n": key, "v": value}
+            )
+
+    def update_cfield(self, key: str, value: str) -> None:
+        """
+        Updates a custom field by key.
+
+        :param key: A custom field key (name).
+        :type key: :py:obj:`str`
+        :param value: A string.
+        :type value: :py:obj:`str`
+        :raises WialonError: If something went wrong with a Wialon API call.
+        :returns: Nothing.
+        :rtype: :py:obj:`None`
+
+        """
+        field_id: int | None = self._get_cfield_id(key)
+
+        if field_id is not None:
+            self.session.wialon_api.item_update_custom_field(
+                **{
+                    "itemId": self.id,
+                    "id": field_id,
+                    "callMode": "update",
+                    "n": key,
+                    "v": value,
+                }
+            )
+        else:
+            self.session.wialon_api.item_update_custom_field(
+                **{"itemId": self.id, "callMode": "create", "n": key, "v": value}
+            )
+
+    def _get_cfield_id(self, key: str) -> int | None:
+        """
+        Returns a custom field id by key.
+
+        :param key: A custom field key (name).
+        :type key: :py:obj:`str`
+        :raises WialonError: If something went wrong with a Wialon API call.
+        :returns: The custom field's id as an integer, if found.
+        :rtype: :py:obj:`int` | :py:obj:`None`
+
+        """
+        fields = (
+            self.session.wialon_api.core_search_item(
+                **{"id": str(self.id), "flags": flags.DataFlag.UNIT_CUSTOM_FIELDS}
+            )
+            .get("item", {})
+            .get("flds", {})
+            .values()
+        )
+
+        for field in fields:
+            if field["n"] == key:
+                return int(field["id"])
+
+    def _get_afield_id(self, key: str) -> int | None:
+        """
+        Returns an admin field id by key.
+
+        :param key: An admin field key (name).
+        :type key: :py:obj:`str`
+        :raises WialonError: If something went wrong with a Wialon API call.
+        :returns: The admin field's id as an integer, if found.
+        :rtype: :py:obj:`int` | :py:obj:`None`
+
+        """
+        fields = (
+            self.session.wialon_api.core_search_item(
+                **{"id": str(self.id), "flags": flags.DataFlag.UNIT_ADMIN_FIELDS}
+            )
+            .get("item", {})
+            .get("aflds", {})
+            .values()
+        )
+
+        for field in fields:
+            if field["n"] == key:
+                return int(field["id"])
+
     @property
     def name(self) -> str:
         """Name of the Wialon object."""
@@ -94,7 +286,7 @@ class WialonBase:
     @property
     def id(self) -> int | None:
         """
-        A unique Wialon ID.
+        A Wialon object id.
 
         :type: :py:obj:`int` | :py:obj:`None`
 
@@ -102,203 +294,28 @@ class WialonBase:
 
         return int(self._id) if self._id else None
 
-    def has_access(self, other: Type["WialonBase"]) -> bool:
+    @property
+    def afields(self) -> dict[str, str]:
         """
-        Checks if this Wialon object has access to ``other``.
+        Admin fields for the Wialon object.
 
-        :type: :py:obj:`bool`
+        :type: :py:obj:`dict`
 
         """
-        response = self.session.wialon_api.core_check_accessors(
-            **{"items": [other.id], "flags": False}
+        response = self.session.wialon_api.core_search_item(
+            **{"id": self.id, "flags": flags.DataFlag.UNIT_ADMIN_FIELDS}
         )
-        return True if self.id in response.keys() else False
-
-    @abstractmethod
-    def create(self, *args, **kwargs) -> int | None:
-        """Creates a Wialon object and returns its id."""
-        raise NotImplementedError("Subclasses must implement this method.")
-
-    def rename(self, new_name: str) -> None:
-        """
-        Renames the Wialon object to the new name.
-
-        :param new_name: A new name.
-        :type new_name: :py:obj:`str`
-        :raises WialonError: If something goes wrong with Wialon.
-        :returns: Nothing.
-        :rtype: :py:obj:`None`
-
-        """
-        self.session.wialon_api.item_update_name(
-            **{"itemId": self.id, "name": new_name}
-        )
-        self._name = new_name
-
-    def get_cfield_id(self, key: str) -> int | None:
-        """
-        Returns a custom field id by its key, if it exists.
-
-        :param key: A custom field key.
-        :type key: :py:obj:`str`
-        :returns: A custom field id, if any.
-        :rtype: :py:obj:`int` | :py:obj:`None`
-
-        """
-        cfields = (
-            self.session.wialon_api.core_search_item(
-                **{"id": str(self.id), "flags": flags.DataFlag.UNIT_CUSTOM_FIELDS}
-            )
-            .get("item", {})
-            .get("flds", {})
-            .values()
-        )
-        for field in cfields:
-            if field["n"] == key:
-                return int(field["id"])
-
-    def get_afield_id(self, key: str) -> int | None:
-        """
-        Returns an admin field id by its key, if it exists.
-
-        :param key: An admin field key.
-        :type key: :py:obj:`str`
-        :returns: An admin field id, if any.
-        :rtype: :py:obj:`int` | :py:obj:`None`
-
-        """
-        afields = (
-            self.session.wialon_api.core_search_item(
-                **{"id": str(self.id), "flags": flags.DataFlag.UNIT_ADMIN_FIELDS}
-            )
-            .get("item", {})
-            .get("aflds", {})
-            .values()
-        )
-        for field in afields:
-            if field["n"] == key:
-                return int(field["id"])
-
-    def add_afield(self, key: str, value: str) -> None:
-        """
-        Adds an admin field to the Wialon object.
-
-        :param key: A key (name) for the admin field.
-        :type key: :py:obj:`str`
-        :param value: A value for the admin field.
-        :type value: :py:obj:`str`
-        :raises WialonError: If something goes wrong with Wialon.
-        :returns: Nothing.
-        :rtype: :py:obj:`None`
-
-        """
-        self.session.wialon_api.item_update_admin_field(
-            **{"itemId": self.id, "id": 0, "callMode": "create", "n": key, "v": value}
-        )
-
-    def update_afield(self, id: int, key: str, value: str) -> None:
-        """
-        Updates an admin field by id.
-
-        :param id: The admin field id.
-        :type id: :py:obj:`int`
-        :param key: A key (name) for the admin field.
-        :type key: :py:obj:`str`
-        :param value: A value for the admin field.
-        :type value: :py:obj:`str`
-        :raises WialonError: If something goes wrong with Wialon.
-        :returns: Nothing.
-        :rtype: :py:obj:`None`
-
-        """
-        self.session.wialon_api.item_update_admin_field(
-            **{"itemId": self.id, "id": id, "callMode": "update", "n": key, "v": value}
-        )
-
-    def add_cfield(self, key: str, value: str) -> None:
-        """
-        Adds a custom field to the Wialon object.
-
-        :param key: A key (name) for the custom field.
-        :type key: :py:obj:`str`
-        :param value: A value for the custom field.
-        :type value: :py:obj:`str`
-        :raises WialonError: If something goes wrong with Wialon.
-        :returns: Nothing.
-        :rtype: :py:obj:`None`
-
-        """
-        self.session.wialon_api.item_update_custom_field(
-            **{"itemId": self.id, "id": 0, "callMode": "create", "n": key, "v": value}
-        )
-
-    def update_cfield(self, id: int, key: str, value: str) -> None:
-        """
-        Updates a custom field by id.
-
-        :param id: The admin field id.
-        :type id: :py:obj:`int`
-        :param field: A key (name) for the custom field.
-        :type field: :py:obj:`str`
-        :param field: A value for the custom field.
-        :type field: :py:obj:`str`
-        :raises WialonError: If something goes wrong with Wialon.
-        :returns: Nothing.
-        :rtype: :py:obj:`None`
-
-        """
-        self.session.wialon_api.item_update_custom_field(
-            **{"itemId": self.id, "id": id, "callMode": "update", "n": key, "v": value}
-        )
-
-    def add_cproperty(self, key: str, value: str) -> None:
-        """
-        Adds a custom property to the Wialon object.
-
-        :param key: A key (name) for the custom property.
-        :type key: :py:obj:`str`
-        :param value: A value for the custom property.
-        :type value: :py:obj:`str`
-        :raises WialonError: If something goes wrong with Wialon.
-        :returns: Nothing.
-        :rtype: :py:obj:`None`
-
-        """
-        self.session.wialon_api.item_update_custom_property(
-            **{"itemId": self.id, "name": key, "value": value}
-        )
-
-    def add_profile_field(self, key: str, value: str) -> None:
-        """
-        Adds a profile field to the Wialon object.
-
-        :param key: A key (name) for the profile field.
-        :type key: :py:obj:`str`
-        :param value: A value for the profile field.
-        :type value: :py:obj:`str`
-        :raises WialonError: If something goes wrong with Wialon.
-        :returns: Nothing.
-        :rtype: :py:obj:`None`
-
-        """
-        self.session.wialon_api.item_update_profile_field(
-            **{"itemId": self.id, "n": key, "v": value}
-        )
-
-    def delete(self) -> None:
-        """
-        Deletes the Wialon object.
-
-        :raises WialonError: If something goes wrong with Wialon.
-        :returns: Nothing.
-        :rtype: :py:obj:`None`
-
-        """
-        self.session.wialon_api.item_delete_item(**{"itemId": self.id})
+        fields = response.get("item", {}).get("aflds") if response is not None else {}
+        return {field["n"]: field["v"] for field in fields.values()} if fields else {}
 
     @property
     def cfields(self) -> dict[str, str]:
-        """Custom fields for the Wialon object."""
+        """
+        Custom fields for the Wialon object.
+
+        :type: :py:obj:`dict`
+
+        """
         response = self.session.wialon_api.core_search_item(
             **{"id": self.id, "flags": flags.DataFlag.UNIT_CUSTOM_FIELDS}
         )
@@ -306,10 +323,15 @@ class WialonBase:
         return {field["n"]: field["v"] for field in fields.values()} if fields else {}
 
     @property
-    def afields(self) -> dict[str, str]:
-        """Admin fields for the Wialon object."""
+    def pfields(self) -> dict[str, str]:
+        """
+        Profile fields for the Wialon object.
+
+        :type: :py:obj:`dict`
+
+        """
         response = self.session.wialon_api.core_search_item(
-            **{"id": self.id, "flags": flags.DataFlag.UNIT_ADMIN_FIELDS}
+            **{"id": self.id, "flags": flags.DataFlag.UNIT_PROFILE_FIELDS}
         )
-        fields = response.get("item", {}).get("aflds") if response is not None else {}
+        fields = response.get("item", {}).get("pflds") if response is not None else {}
         return {field["n"]: field["v"] for field in fields.values()} if fields else {}
